unit form_AdvancedCopy;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, Spin64, frame_Location, ComCtrls;

type
  Tfrm_AdvancedCopy = class(TForm)
    pcMain: TPageControl;
    tsSource: TTabSheet;
    Label2: TLabel;
    tsDestination: TTabSheet;
    Label6: TLabel;
    tsOptions: TTabSheet;
    Label1: TLabel;
    Label7: TLabel;
    lblLimitBytes: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    se64StartOffset: TSpinEdit64;
    ckLimitCopy: TCheckBox;
    se64MaxBytesToCopy: TSpinEdit64;
    seBlockSize: TSpinEdit;
    lblProgress: TLabel;
    Label4: TLabel;
    pbCopy: TButton;
    pbCancel: TButton;
    fmeLocationSrc: Tfme_Location;
    fmeLocationDest: Tfme_Location;
    pbClose: TButton;
    procedure pbCloseClick(Sender: TObject);
    procedure pbAboutClick(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure ckLimitCopyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure pbCopyClick(Sender: TObject);
  private
    Processing: boolean;
    CancelFlag: boolean;

    procedure EnableDisableControls();

    procedure ProgressCallback(progress: int64; var cancel: boolean);

  public
    { Public declarations }
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
  About_U;

const
  SECTOR_SIZE = 512;


procedure Tfrm_AdvancedCopy.ckLimitCopyClick(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure Tfrm_AdvancedCopy.FormShow(Sender: TObject);
begin
  lblProgress.Caption := 'n/a';

  pcMain.activepage := tsSource;

  pbCancel.Enabled := FALSE;
  fmeLocationSrc.EnableDisableControls();
  fmeLocationDest.EnableDisableControls();

  EnableDisableControls();

end;

procedure Tfrm_AdvancedCopy.pbAboutClick(Sender: TObject);
begin
  ShowAboutDialog();
end;

procedure Tfrm_AdvancedCopy.pbCancelClick(Sender: TObject);
begin
  CancelFlag := TRUE;

end;

procedure Tfrm_AdvancedCopy.pbCloseClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure Tfrm_AdvancedCopy.pbCopyClick(Sender: TObject);
var
  allOK: boolean;
  length: int64;
  prevCursor: TCursor;
  locationSrc, locationDest: string;
begin
  allOK := TRUE;
  CancelFlag := FALSE;

  locationSrc:= fmeLocationSrc.GetLocation();
  locationDest:= fmeLocationDest.GetLocation();

  if allOK then
    begin
    if (locationSrc = '') then
      begin
      SDUMessageDlg(
                 'Please specify where you wish to copy data from',
                 mtError,
                 [mbOK],
                 0
                );
      allOK := FALSE;
      end;
    end;

  if allOK then
    begin
    if (locationDest = '') then
      begin
      SDUMessageDlg(
                 'Please specify where you wish to copy data to',
                 mtError,
                 [mbOK],
                 0
                );
      allOK := FALSE;
      end;
    end;

  if allOK then
    begin
    if (locationSrc = locationDest) then
      begin
      SDUMessageDlg(
                 'Source location cannot be the same as the destination',
                 mtError,
                 [mbOK],
                 0
                );
      allOK := FALSE;
      end;
    end;

  if allOK then
    begin
    allOK := (SDUMessageDlg(
                   'About to copy FROM:'+SDUCRLF+
                   SDUCRLF+
                   locationSrc+SDUCRLF+
                   SDUCRLF+
                   'TO:'+SDUCRLF+
                   SDUCRLF+
                   locationDest+SDUCRLF+
                   SDUCRLF+
                   'Do you wish to proceed?',
                   mtConfirmation, [mbYes, mbNo], 0) = mrYes);
    end;

  if allOK then
    begin
    allOK := (SDUMessageDlg(
                   'WARNING:'+SDUCRLF+
                   SDUCRLF+
                   'THIS WILL OVERWRITE:'+SDUCRLF+
                   SDUCRLF+
                   fmeLocationDest.GetLocation()+SDUCRLF+
                   SDUCRLF+
                   'Are you sure you wish to proceed?',
                   mtWarning, [mbYes, mbNo], 0) = mrYes);
    end;

  if allOK then
    begin
    length := -1;
    if (ckLimitCopy.checked) then
      begin
      length := se64MaxBytesToCopy.Value;
      end;


    prevCursor := Screen.Cursor;
    Screen.Cursor := crAppStart;
    try
      Processing := TRUE;
      EnableDisableControls();

      SDUCopyFile(
           locationSrc,
           locationDest,
           se64StartOffset.value,
           length,
           seBlockSize.value,
           ProgressCallback
          );

    finally
      Processing := FALSE;
      EnableDisableControls();

      Screen.Cursor := prevCursor;
    end;

    end;

end;

procedure Tfrm_AdvancedCopy.EnableDisableControls();
begin

  SDUEnableControl(pcMain, not(Processing));
  SDUEnableControl(pbCopy, not(Processing));

  SDUEnableControl(pbCancel, Processing);
  SDUEnableControl(pbClose, not(Processing));

  if not(Processing) then
    begin
    SDUEnableControl(se64MaxBytesToCopy, ckLimitCopy.checked);
    SDUEnableControl(lblLimitBytes, ckLimitCopy.checked);
    end;

end;


procedure Tfrm_AdvancedCopy.ProgressCallback(progress: int64; var cancel: boolean);
begin
  lblProgress.caption := inttostr(progress);
  Application.ProcessMessages();
  cancel := CancelFlag;
end;

END.

