unit form_BackupRestore;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  SDUProgress_U;

type
  TBackupRestoreDlgType = (opBackup, opRestore);

  TfrmBackupRestore = class(TForm)
    pbGo: TButton;
    pbCancel: TButton;
    pnlDrive: TPanel;
    pnlFile: TPanel;
    lblFile: TLabel;
    edFilename: TEdit;
    pbBrowse: TButton;
    cbDrive: TComboBox;
    lblDrive: TLabel;
    pnlSrc: TPanel;
    pnlDest: TPanel;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    procedure pbGoClick(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FDlgType: TBackupRestoreDlgType;
    Processing: boolean;
    ProgressDlg: TSDUProgress_F;

    procedure EnableDisableControls();
  public
    property DlgType: TBackupRestoreDlgType read FDlgType write FDlgType;

    procedure ProgressCallback(progress: int64; var cancel: boolean);

  end;


implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
{$WARNINGS OFF}  // Useless warning about platform - don't care about it
  FileCtrl,
{$WARNINGS ON}
  AppGlobals;

procedure TfrmBackupRestore.FormShow(Sender: TObject);
begin
  Processing := FALSE;

  if (DlgType = opBackup) then
    begin
    self.Caption := 'Backup';
    pbGo.Caption := 'Backup';

    lblDrive.Caption := 'Please select the drive to be backed up:';
    lblFile.Caption := 'Please specify where the backup is to be stored:';

    pnlDrive.left   := pnlSrc.left;
    pnlDrive.top    := pnlSrc.top;
    pnlFile.left    := pnlDest.left;
    pnlFile.top     := pnlDest.top;
    end
  else
    begin
    self.Caption := 'Restore';
    pbGo.Caption := 'Restore';

    lblFile.Caption := 'Please specify the location of the backup to be restored:';
    lblDrive.Caption := 'Please select the drive to be restored:';

    pnlDrive.left   := pnlDest.left;
    pnlDrive.top    := pnlDest.top;
    pnlFile.left    := pnlSrc.left;
    pnlFile.top     := pnlSrc.top;
    end;

  pnlDrive.BevelInner := bvNone;
  pnlDrive.BevelOuter := bvNone;
  pnlDrive.Caption := '';

  pnlFile.BevelInner := bvNone;
  pnlFile.BevelOuter := bvNone;
  pnlFile.Caption := '';

  pnlSrc.BevelInner := bvNone;
  pnlSrc.BevelOuter := bvNone;
  pnlSrc.Caption := '';
  pnlSrc.visible := FALSE;

  pnlDest.BevelInner := bvNone;
  pnlDest.BevelOuter := bvNone;
  pnlDest.Caption := '';
  pnlDest.visible := FALSE;

  PopulateRemovableDrives(cbDrive);

end;


procedure TfrmBackupRestore.pbBrowseClick(Sender: TObject);
begin
  if (DlgType = opBackup) then
    begin
    SDUOpenSaveDialogSetup(SaveDialog1, edFilename.text);

    if SaveDialog1.Execute then
      begin
      edFilename.text := SaveDialog1.Filename;
      end;
    end
  else
    begin
    SDUOpenSaveDialogSetup(OpenDialog1, edFilename.text);

    if OpenDialog1.Execute then
      begin
      edFilename.text := OpenDialog1.Filename;
      end;

    end;

end;

procedure TfrmBackupRestore.pbCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmBackupRestore.pbGoClick(Sender: TObject);
var
  allOK: boolean;
  locationSrc: string;
  locationDest: string;
  errMsg: string;
  size: LARGE_INTEGER;
  driveItem: string;
  drive: char;
  driveDevice: string;
  prevCursor: TCursor;
  confirmMsg: string;
  opType: string;
begin
  allOK := TRUE;

  if (cbDrive.ItemIndex < 0) then
    begin
    if (DlgType = opBackup) then
      begin
      errMsg := 'Please specify which drive is to be backed up.';
      end
    else
      begin
      errMsg := 'Please specify drive to restore.';
      end;

    SDUMessageDlg(errMsg, mtError, [mbOK], 0);

    allOK := FALSE;
    end;

  if (edFilename.Text = '') then
    begin
    if (DlgType = opBackup) then
      begin
      errMsg := 'Please specify where to store the backup.';
      end
    else
      begin
      errMsg := 'Please specify which backup to restore.';
      end;

    SDUMessageDlg(errMsg, mtError, [mbOK], 0);

    allOK := FALSE;
    end;

  if allOK then
    begin
    driveItem := cbDrive.Items[cbDrive.Itemindex];
    drive := driveItem[1];
    driveDevice := '\\.\'+drive+':';

    if (DlgType = opBackup) then
      begin
      locationSrc:= driveDevice;
      locationDest:= edFilename.Text;

      size := SDUGetPartitionSize(drive);

      confirmMsg:= 'Are you sure you wish to backup drive '+drive+': to:'+SDUCRLF+
                   SDUCRLF+
                   locationDest;
      end
    else
      begin
      locationSrc:= edFilename.Text;
      locationDest:= driveDevice;

      size := SDUGetFileSize(locationSrc);

      confirmMsg:= 'Are you sure you wish to restore '+SDUCRLF+
                   SDUCRLF+
                   locationSrc+SDUCRLF+
                   SDUCRLF+
                   'to drive '+drive+':?';
      end;

    end;

  if allOK then
    begin
    allOK := (SDUMessageDlg(confirmMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
    end;

  if allOK then
    begin
    prevCursor := Screen.Cursor;
    Screen.Cursor := crAppStart;
    ProgressDlg := TSDUProgress_F.Create(self);
    try
      Processing := TRUE;
      EnableDisableControls();

      ProgressDlg.i64Min := 0;
      ProgressDlg.i64Max := size.QuadPart;
      ProgressDlg.i64Position := 0;

      ProgressDlg.ShowTimeRemaining := TRUE;

      ProgressDlg.Show();

      if SDUCopyFile(
                  locationSrc,
                  locationDest,
                  0,
                  -1,  // Do all of it
                  STD_BUFFER_SIZE,
                  ProgressCallback
                 ) then
        begin
        if (DlgType = opBackup) then
          begin
          opType := 'Backup';
          end
        else
          begin
          opType := 'Restore';
          end;

        SDUMessageDlg(
                      opType+' completed successfully.',
                      mtInformation,
                      [mbOK],
                      0
                     );
                     
        ModalResult := mrOK;
        end;

    finally
      Processing := FALSE;
      EnableDisableControls();

      ProgressDlg.Free();

      Screen.Cursor := prevCursor;
    end;

    end;

end;

procedure TfrmBackupRestore.EnableDisableControls();
begin
  SDUEnableControl(cbDrive, not(Processing));
  SDUEnableControl(edFilename, not(Processing));
  SDUEnableControl(pbBrowse, not(Processing));
  SDUEnableControl(pbGo, not(Processing));
  SDUEnableControl(pbCancel, not(Processing));

end;

procedure TfrmBackupRestore.ProgressCallback(progress: int64; var cancel: boolean);
begin
  ProgressDlg.i64Position := progress;
  Application.ProcessMessages();
  cancel := ProgressDlg.Cancel;
end;


END.

